;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x9e132e9dcabdaea42e1c129096dece9e . "./code.cg2js/cg-acache.fasl")
(#xa3485070ba820db0484e43c58d23dbcd . "./code.cg2js/cg-base.fasl")
(#xbc895050bfe71a17ab14642f64fa3abe . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x24871daec24195c0e5174d65219ac0af . "./code.cg2js/cg-bitmap-pane.fasl")
(#x114cc6fd56e6b142a2fa91891f8fc5fc . "./code.cg2js/cg-bitmap-stream.fasl")
(#x26033556437dd0ae5308a96621c60367 . "./code.cg2js/cg-button.fasl")
(#x032a631a2c61e8f2ebd264b21c3731fe . "./code.cg2js/cg-calendar.fasl")
(#x44f2f5ad731a45dcf624c47af670c037 . "./code.cg2js/cg-caret.fasl")
(#x62cbf8d2b5d1eddc209c3b60c4db507d . "./code.cg2js/cg-chart-or-plot.fasl")
(#x486b596885ae905945eaf4c48c0a95d3 . "./code.cg2js/cg-chart-widget.fasl")
(#x1bbb1fd2b5f9f9e97afe4cc8f1cb7358 . "./code.cg2js/cg-check-box.fasl")
(#x1532d39b6fd4320bc91e92162c034dfe . "./code.cg2js/cg-choice-list.fasl")
(#x0258064f46e759d896bc94f799c4bc11 . "./code.cg2js/cg-choose-printer.fasl")
(#x0f7514f427343c4441ed7712e4524e31 . "./code.cg2js/cg-class-grid.fasl")
(#x89ebc818c6d714f3d9eb9318b31ce073 . "./code.cg2js/cg-class-slot-grid.fasl")
(#xa6ae19ea23d5f538f2c2588e53824cee . "./code.cg2js/cg-class-support.fasl")
(#x25ad2e07ff3295d75dfd247632926f1a . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x93e1dbfe89fa7ed6724945e827f070e4 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x88528ce8a9ee48eb5d4c865e923f53d6 . "./code.cg2js/cg-clipboard.fasl")
(#xc07e138597aa51e64a0e526d000eb680 . "./code.cg2js/cg-color-dialog.fasl")
(#xe15f67de9c55072d0a143b76fa1e3569 . "./code.cg2js/cg-combo-box.fasl")
(#x4a6a56888b40a367938d7451c2529703 . "./code.cg2js/cg-common-control.fasl")
(#x8f7d345a411992115e009802f65acf26 . "./code.cg2js/cg-comtab.fasl")
(#x56199bbf0b7c6c5c67ac6150d83a0a40 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#xcbc7669c988e528698ea7102c8b8c427 . "./code.cg2js/cg-curve.fasl")
(#xf969a61298f24f461450b2148d53f366 . "./code.cg2js/cg-dde-utils.fasl")
(#x8aa525463b3684cac62e69861775c29c . "./code.cg2js/cg-dev.fasl")
(#x806b7e8a04e20026a67d3a492ad6496b . "./code.cg2js/cg-dialog-item.fasl")
(#x5e32d1bcee11ae92e23015edc7484e02 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xb127d32ce619247792398a749d6d72fc . "./code.cg2js/cg-directory-dialog.fasl")
(#xa053eb021bf039208fab1257ecee152f . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#xad18c32f675b25e63a9d3cd14ed739c7 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x8b96dc67146ad279516f549f804d17ec . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xe55bf4617ac7bc328585e8ea4544ba35 . "./code.cg2js/cg-drawable.fasl")
(#x9643ea8f8aacddf98e5a9b744168a3e8 . "./code.cg2js/cg-dropping-outline.fasl")
(#xfbf0a32e7c3cd67260a6e8b7fc83a6d4 . "./code.cg2js/cg-edit-in-place.fasl")
(#x5de3e963ad1ecf9764b525fb06e6d750 . "./code.cg2js/cg-editable-text.fasl")
(#x3bafa7c6b4ef345a680aa01237c77954 . "./code.cg2js/cg-file-dialog.fasl")
(#x763ab522f22affd54e3ba1e3f1fba329 . "./code.cg2js/cg-fill-texture.fasl")
(#x25669ec4d9ad1e3b3c9ccce2f219c932 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x02c20338fd7c61e2f363b7aa607ae52a . "./code.cg2js/cg-font-dialog.fasl")
(#xb949a8f0d252e35d36830a0096355611 . "./code.cg2js/cg-gesture-emulation.fasl")
(#x6f014937b897c04d09f4065362d5fd66 . "./code.cg2js/cg-get-pixmap.fasl")
(#x7ec410003838d3aa104331887d399ebf . "./code.cg2js/cg-get-position.fasl")
(#x2d7521b8813c08dd4e7461c8a4a10105 . "./code.cg2js/cg-graphics-context.fasl")
(#x25d6a017d87c8cf4f9421338e0d2bb3a . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x45754f2f101f2c9986705ca064503b0e . "./code.cg2js/cg-grid-widget.fasl")
(#xe7d50293e11ee182dca76b82ecdbb3cf . "./code.cg2js/cg-group-box.fasl")
(#x5a0517ff0b70cfe73f644948b6efa7c0 . "./code.cg2js/cg-header-control.fasl")
(#xa5ecfe5a0d165368f154d4ade842203d . "./code.cg2js/cg-hotspot.fasl")
(#x24259dcfba9d2930fb1bc335546b9862 . "./code.cg2js/cg-html-dialog.fasl")
(#x8d2ae181b2679044afe822552ec3863a . "./code.cg2js/cg-html-widget.fasl")
(#xcf7b167cd752f94e29d369c4f2ebc3b9 . "./code.cg2js/cg-icon-pixmap.fasl")
(#x7045a762211477a053e51754810bfe55 . "./code.cg2js/cg-icon.fasl")
(#x062c97560097829fcc0a6553adfeede0 . "./code.cg2js/cg-ie.fasl")
(#xf3fe1619ded5e0360703332cb05df68f . "./code.cg2js/cg-intersect-posbox.fasl")
(#xed7b156abac0825f87abe3db7254cbb8 . "./code.cg2js/cg-intersect.fasl")
(#xb2e0cfe9be1c62da3386f0e191772bce . "./code.cg2js/cg-item-list.fasl")
(#x30fe2c43eab7583fb3a9e98cd5abb799 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x119272a3c1dfd76bd9055502d55f2f94 . "./code.cg2js/cg-lamp.fasl")
(#x6feedc61a52ad8e52a9f629e09b34a49 . "./code.cg2js/cg-layout.fasl")
(#x5a5ec753d0933250d3b1c6b7f039e866 . "./code.cg2js/cg-lettered-menu.fasl")
(#x8753d5e3b66ce11b759198322ef23e26 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#xeeea2d79f3933ed84ae278f65b650505 . "./code.cg2js/cg-lisp-text.fasl")
(#x196bbdbbeeebafc104d0da3aecc476b0 . "./code.cg2js/cg-lisp-widget.fasl")
(#x6e177535fdf58de71994c798fe19f180 . "./code.cg2js/cg-list-view.fasl")
(#x73c636b2d2abff194696d1d875a5b23d . "./code.cg2js/cg-mci.fasl")
(#xa110af8d68d258a731d2d6a746cc7135 . "./code.cg2js/cg-menu-tooltip.fasl")
(#x2ec03098a2bb7b3d9f59620d65f63103 . "./code.cg2js/cg-menu.fasl")
(#xae113d9548f14df27da9ae9c12f527d6 . "./code.cg2js/cg-message-dialog.fasl")
(#x816929ed3384519d2398fee9c794c7f7 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x08aa9a64d59854efa5ee566ca7b896b1 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#xd478e5ddc4c9a2cd906385663859bad5 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x45a1e80770127c25d0d765deae1f708a . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x3279904445ef7d1110415105081d1bcd . "./code.cg2js/cg-multi-picture-button.fasl")
(#x32efcda57d4181fcc65ea9564f9ab49f . "./code.cg2js/cg-nodes.fasl")
(#x8e72a72cb78efbf4d01bb638d4fb70f1 . "./code.cg2js/cg-object-editor-layout.fasl")
(#x49f104fcaf37fe38273a08af7bfc5178 . "./code.cg2js/cg-object-editor.fasl")
(#x5b1d785d0d2c46895096919db5b79efa . "./code.cg2js/cg-ocx.fasl")
(#x0537ad690cdc5bd1ce19d8eee322073f . "./code.cg2js/cg-os-widget.fasl")
(#x620ba2a51e4c9f7f33c8c42ee3d4ce8e . "./code.cg2js/cg-os-window.fasl")
(#x8b55b246fbe2ec38f56bc77aefca5d42 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x1c8be9a229c968005599d5d48bdf0a28 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x94ad326e8a5623c14de016f24a6045da . "./code.cg2js/cg-outline.fasl")
(#xebe24c7eb380fd21531626b958bb7f50 . "./code.cg2js/cg-palette.fasl")
(#xfe6a08b9b3e06b316adbe52a39002bc2 . "./code.cg2js/cg-paren-matching.fasl")
(#x798bcaef5fa4d6ad28460e13adf97f47 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x6472c2e22ed192c6fa3fc34ae0eaad6b . "./code.cg2js/cg-picture-widget.fasl")
(#x9c238a05f2f41b39e05547df138ebd79 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x3f12255ef444f95dd42ded769f111e0c . "./code.cg2js/cg-pixmap-printing.fasl")
(#x15aa578eb2b9427c509f5ee75393640c . "./code.cg2js/cg-pixmap-rotate.fasl")
(#xbad94bd1e337c73d73044ec4ba1c2fab . "./code.cg2js/cg-pixmap-widget.fasl")
(#xe95999c28e3460455ab6a3ba83998fd9 . "./code.cg2js/cg-pixmap.fasl")
(#x1fbb23ad623c288997a7e0a54634f224 . "./code.cg2js/cg-player.fasl")
(#x5f25813baaba941208f4c533c17c53d3 . "./code.cg2js/cg-plot-widget.fasl")
(#xcb938eb643dd629daf70286efcfc5a5f . "./code.cg2js/cg-printing.fasl")
(#x78e443296e28bd578a5565669c96d498 . "./code.cg2js/cg-progress-indicator.fasl")
(#x8b5491ff04cac1963fc9824f539dad34 . "./code.cg2js/cg-project-window.fasl")
(#x164437378e6556dbbe3159bab38ff8f7 . "./code.cg2js/cg-property.fasl")
(#x0d75ade2a923392370f5e2793ae87863 . "./code.cg2js/cg-radio-button.fasl")
(#x0e3c6060fbd5be185df8338bd466b531 . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#x71398722818d8311e029838078d1f6f4 . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#xd40f64d2e7ce1b2fcb0cf5f3bb2dfa3f . "./code.cg2js/cg-rich-edit-pane.fasl")
(#x22a3c7d1e6349ba092d6c88426802b26 . "./code.cg2js/cg-rich-edit.fasl")
(#xe648bb861841ddf647b7c7f11162fd2c . "./code.cg2js/cg-sample-file-menu.fasl")
(#x85057e50fb91e2e772bb29d0766fad07 . "./code.cg2js/cg-scaling-stream.fasl")
(#x2c0c18566437695c53a2e6e6d114802a . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#xe6c30b53b614f1feda0d4654e80c8428 . "./code.cg2js/cg-scroll-bar.fasl")
(#x37c5a541f17e458041881ca736d8048b . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x90c41a3b6319abb535f5ed97c7c1d628 . "./code.cg2js/cg-selected-object.fasl")
(#x32052f5f0e2ce07848951f6324094b25 . "./code.cg2js/cg-shortcut-menu.fasl")
(#x71c2b76beec526454320d0a7e47c07f5 . "./code.cg2js/cg-split-bar.fasl")
(#x8108b3f09f064472681d52ff3bb32cf1 . "./code.cg2js/cg-static-text.fasl")
(#xfe324a552968011346f470c20f614d33 . "./code.cg2js/cg-status-bar.fasl")
(#x51c3f3b96bced1c2d916311b4ccf8d3a . "./code.cg2js/cg-string-dialog.fasl")
(#xf8bb9d43ca04bccac61778c6ea5fdc18 . "./code.cg2js/cg-tab-control.fasl")
(#x7f89a3323e0b19116b14a1c5c892107b . "./code.cg2js/cg-template-string.fasl")
(#x067224cb76c360857ce44247b8bb5c58 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x99d2b6a696cfbe439affa6e7299bbada . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x0986ec2ef30865d0464dd116c87ff500 . "./code.cg2js/cg-text-edit-pane.fasl")
(#x953b86edd25eba6192e8cb83f0c134c9 . "./code.cg2js/cg-text-or-combo.fasl")
(#x8070df641833e49dd5e2a374aa0c0053 . "./code.cg2js/cg-text-widget.fasl")
(#x25afb41a16cd5f728fdf3f46d2eb935f . "./code.cg2js/cg-timer.fasl")
(#xa39df88c349ef1e3122935c9eea7b55b . "./code.cg2js/cg-toggling-widget.fasl")
(#x4b1d1e3b1f2eebd62453d2076ab47b5a . "./code.cg2js/cg-toolbar.fasl")
(#x28fcfa69253db43ee1d2e7c288f34c29 . "./code.cg2js/cg-tooltip.fasl")
(#x948571ac3aa2c0dbfc8bc79cbed64eab . "./code.cg2js/cg-touch.fasl")
(#x708fdbbe64b15dce9c433842a4753ec3 . "./code.cg2js/cg-trackbar.fasl")
(#x5506394066899c551c1639ba8954507c . "./code.cg2js/cg-tray.fasl")
(#x09963664821bdf77df65135114f191bb . "./code.cg2js/cg-up-down-control.fasl")
(#x21104f9a5ce1f6ee489607f399c8c308 . "./code.cg2js/cg-utility-dialog.fasl")
(#xc68490e7e26381bffca94e34cfa35eb0 . "./code.cg2js/cg-web-browser-dde.fasl")
(#x95ced2e3cc7d4d472e708fbf38639e7d . "./code.cg2js/cg-web-browser.fasl")
(#x528178b847d6c89b01fbfda6ad5c6778 . "./code.cg2js/cg-wrap-string.fasl")
(#xe74bf109555e05f83bcfaeed24e0cff7 . "./code.cg2js/cg-yes-no-list.fasl")
(#x922e5a7dd45210498ea09263ed791f51 . "./code.cg2js/cg.fasl")
(#xe39d2e52a3d237023464724ec3018701 . "./code.cg2js/cg-yes-no-string.fasl")
(#x0fb69db80ea64ec29cedeee7b841b3f4 . "./code.idejs/ide-acache.fasl")
(#xbc69f2bfb5854bdfdb4eaa0e463a2f92 . "./code.idejs/ide-apropos.fasl")
(#xd49ed5159bcaf665bbdc9bed9993f9cd . "./code.idejs/ide-base.fasl")
(#x0bf8bbf31b6a9df6f8c015ca982285b0 . "./code.idejs/ide-class-browser.fasl")
(#x602cd524fb4621158aa038b0d798deb4 . "./code.idejs/ide-class-grapher.fasl")
(#xade174957a7d32479507afc2e104f613 . "./code.idejs/ide-clipboard.fasl")
(#x806ced06d964d318b25d78aff6350f04 . "./code.idejs/ide-dev.fasl")
(#x66c6ac9921680a3473b2c0f8685a203b . "./code.idejs/ide-editor.fasl")
(#x205ecf3fb8700651e4e63635d9c33c58 . "./code.idejs/ide-find-definitions.fasl")
(#x519c74865ff1b111011347783afd0cd8 . "./code.idejs/ide-find-in-files.fasl")
(#xd88139acee12ee7887d0fb4676206d00 . "./code.idejs/ide-form.fasl")
(#x01aca51c11133a6795bef04585b14593 . "./code.idejs/ide-grapher.fasl")
(#xfc2012557576624a2b14c328c3b8894d . "./code.idejs/ide-inspector.fasl")
(#x11b7bc1997b31aae9bb8837ab47ff9a4 . "./code.idejs/ide-intro.fasl")
(#x06bb91cd697f8ef3eb9a9315723615d8 . "./code.idejs/ide-keyshort.fasl")
(#x60c0aaeeb5675eef6a57a2f054a0c059 . "./code.idejs/ide-navigator.fasl")
(#x9cb445010ab6606aaece7ba5c1964f39 . "./code.idejs/ide-package-list.fasl")
(#x7218bf8e9b6631fdc7fbba160dd19d31 . "./code.idejs/ide-patch-dialog.fasl")
(#xb81464bddbf752ee814d199970dfebf3 . "./code.idejs/ide-process-browser.fasl")
(#x52f7c2bd71ec1698ad7ab86078e7998f . "./code.idejs/ide-project.fasl")
(#x0618f85521231fc7105715b6f6e3de6a . "./code.idejs/ide-trace.fasl")
(#xcb647ef6163edc9e52fa49d417c040fa . "./code.idejs/ide-window-list.fasl")
(#xa9311f751eb03b70e87ec8293d730853 . "./code.idejs/ide.fasl")
(#xf82fd41d79afb1da19d111da8345005f . "./cgjs.html")
(#x55fea08d92e113f5a8381c271febbc9e . "./ssl-not-found.html")
(#x4e1b21622d7d9cc4e6fc09c7875f8430 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
